% R5Pup MEX EXAMPLE
% calculates absolute pose for 2 images with various rolling shutter effects
% 3D points and 2D image points are provided for the computation
% images are provided for visual verification
% 
% comparison of P3P and R5Pup
%
% a simple version of RANSAC is used for robust estimation
%
% this code demonstrates the use of R5Pup absolute pose solver for Rolling 
% shutter cameras
%
% please refere to the paper
% 
% Cenek Albl, Zuzana Kukelova, Tomas Pajdla;
% Rolling Shutter Absolute Pose Problem With Known Vertical Direction
% The IEEE Conference on Computer Vision and Pattern Recognition (CVPR),
% 2016, pp. 3355-3363
%
% 2016, Cenek Albl, alblcene@cmp.felk.cvut.cz
clear all
%calibration matrix 
K = 1.0e+03 *[1.5442      0    0.9488
                0    1.5447    0.5473
                0         0    0.0010];
img = imread('data/1.jpg'); 
%vertical rolling shutter
rs_direction = 1;

load('data\demo_data');

R_best=[];
w_best=[];
t_best=[];
C_best=[];
u_p_rp5up_best = [];

err_best = Inf;
%calculate R5Pup
idxs  = nchoosek(1:10,5);
tic
for i =1:size(idxs,1)
    [R w t C] = r5pup_mex(g(:,1),X(:,idxs(i,:)),u(:,idxs(i,:)),rs_direction);
    %check reprojection error
    %over all results
    for j=1:size(w,2)
        %skip results with meaningless camera velocities
        wj = w(:,j);
        if(norm(wj)>2)
            continue
        end
        tj = t(:,j);
        Cj = C(:,j);
        Rj = reshape(R(:,j),3,3);
        err=0;
        u_p = [];
        %over all points
        for l=1:28
           %project 3D point using calculated camera pose
           u_p(:,l) = (eye(3)+X_(wj*u(rs_direction+1,l)))*Rj*X(:,l)+Cj+tj*u(rs_direction+1,l);
        end
        u_p = h2a(u_p);
        err = norm(u-u_p(1:2,:));
        if(err<err_best)
            %save best parameters and projections
           R_best = Rj;
           C_best = Cj;
           t_best = tj;
           w_best = wj;
           err_best = err;
           u_p_r5pup_best = u_p(1:2,:);
        end
    end        

end
toc
% draw results
% results from P3P are already in the image
% detected marker points are WHITE
% P3P reprojected marker points are in PINK and a cube projected using
% parameters from P3P is in BLUE
% r5pup results are in RED
warning('off', 'Images:initSize:adjustingMag');
figure
imshow(img);
hold on
%draw the best found r5pup reprojections 
u_r = K*a2h(u_p_r5pup_best);
plot(u_r(1,:),u_r(2,:),'.r','MarkerSize',20);
u_or = h2a(K*a2h(u));
%project a cube using perspective projection and the perspective parameters
%from r5pup
scale = 0.25;
center = [0.5 -0.3 0.5]';
cube = scale*unique([[1 1 1]' [-1 -1 -1]' perms([-1 -1 1])' perms([-1 1 1])' ]','rows')'+repmat(center,1,8);
r5pupP = K*[R_best C_best];
cube_r5pupP = h2a(r5pupP*a2h(cube));
idxs = [1 2 4 3 7 8 6 5 7 5 1 3 4 8 6 2];
plot(cube_r5pupP(1,idxs),cube_r5pupP(2,idxs),'r','LineWidth',3);